package com.jz.dmp.cmdexectool.common.interceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.lang3.StringUtils;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.resource.ResourceHttpRequestHandler;

import com.alibaba.fastjson.JSONObject;
import com.jz.dmp.cmdexectool.common.annotation.ResubmitCheck;
import com.jz.dmp.cmdexectool.common.bean.BaseResponse;
import com.jz.dmp.cmdexectool.common.constant.CommConstant;
import com.jz.dmp.cmdexectool.common.constant.StatuConstant;

/**
 * @ClassName: LogInterceptor
 * @Description: TODO(重复提交校验)
 * @author ybz
 * @date 2021年1月25日
 *
 */
public class ResubmitCheckInterceptor implements HandlerInterceptor {
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
    	
    	if (handler instanceof ResourceHttpRequestHandler) {
			return true;
		}
        
    	HandlerMethod handlerMethod = (HandlerMethod)handler;
    	
    	ResubmitCheck resubmitCheck = handlerMethod.getMethod().getAnnotation(ResubmitCheck.class);
    	if (resubmitCheck!=null) {//判断是否需要进行重复提交校验
			String resubmitToken = request.getHeader(CommConstant.RE_SUBMIT_TOKEN);
			if (resubmitToken==null) {
				resubmitToken = request.getParameter(CommConstant.RE_SUBMIT_TOKEN);
			}
			
			if (StringUtils.isEmpty(resubmitToken)) {//没有上传校验token
				BaseResponse baseResponse = new BaseResponse();
				baseResponse.setCode(StatuConstant.CODE_ERROR_PARAMETER);
				baseResponse.setMessage("重复提交校验token不能为空！");
				response.getWriter().write(JSONObject.toJSONString(baseResponse));
				return false;
			}
			HttpSession session = request.getSession();
			String resubmitTokenSession = (String) session.getAttribute(CommConstant.RE_SUBMIT_TOKEN);
			if (!resubmitToken.equals(resubmitTokenSession)) {//判断是否重复提交
				BaseResponse baseResponse = new BaseResponse();
				baseResponse.setCode(StatuConstant.FAILURE_CODE);
				baseResponse.setMessage("重复提交！");
				response.getWriter().write(JSONObject.toJSONString(baseResponse));
				return false;
			}
			
			//校验通过，置空token
			session.setAttribute(CommConstant.RE_SUBMIT_TOKEN, null);
		}
    	
        return true;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView)
            throws Exception {
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex)
            throws Exception {
    }
}
