package com.jz.dmp.cmdexectool.common.utils;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.ethz.ssh2.ChannelCondition;
import ch.ethz.ssh2.Connection;
import ch.ethz.ssh2.Session;
import ch.ethz.ssh2.StreamGobbler;  
/** 
 * 远程执行linux的shell script 
 * @author hht 
 * @since  V0.1 
 */  
public class RemoteExecuteCommand {  
	
	private static final Logger logger = LoggerFactory.getLogger(RemoteExecuteCommand.class);

    //字符编码默认是utf-8  
    private static String  DEFAULTCHART="UTF-8";  
    private Connection conn;  
    private String ip;  
    private String userName;  
    private String userPwd;  
      
    public RemoteExecuteCommand(String ip, String userName, String userPwd) {  
        this.ip = ip;  
        this.userName = userName;  
        this.userPwd = userPwd;  
    }  
      
    public RemoteExecuteCommand() {  
          
    }  
      
    /** 
     * 远程登录linux的主机 
     * @author Ickes 
     * @since  V0.1 
     * @return 
     *      登录成功返回true，否则返回false 
     */  
    public Boolean login(){  
        boolean flg=false;  
        try {  
            conn = new Connection(ip);  
            conn.connect();//连接  
            flg=conn.authenticateWithPassword(userName, userPwd);//认证  
        } catch (IOException e) {  
            e.printStackTrace();  
        }  
        return flg;  
    }  
    /** 
     * @author Ickes 
     * 远程执行shll脚本或者命令 
     * @param cmd 
     *      即将执行的命令 
     * @return 
     *      命令执行完后返回的结果值:0#XXX,前两位为执行返回值：0#表示执行成功，其他表示失败
     * @since V0.1 
     */  
    public String execute(String cmd){  
        String result="";  
        Session session= null;
        try {  
            if(login()){
                session= conn.openSession();//打开一个会话  
                session.execCommand(cmd);//执行命令  
                result=processStdout(session.getStdout(),DEFAULTCHART);  
                //如果为得到标准输出为空，说明脚本执行出错了 
                if(StringUtils.isBlank(result)){  
                    result=processStdout(session.getStderr(),DEFAULTCHART);  
                }
                result =session.getExitStatus()+"#"+result;
            }  
        } catch (IOException e) {  
            e.printStackTrace(); 
        } finally {
        	if(conn != null) {
        		conn.close();
        	}
        	if(session != null) {
        		session.close();
        	}
        }
        return result;  
    }  
    
    /**
     * 命令是否执行成功
     * @param cmd 要执行的远程命令
     * @return true:执行成功  执行命令
     */
    public boolean isExecuteSuccess(String cmd){  
    	logger.info("[ip="+this.ip+", userName="+this.userName+", password="+this.userPwd+"]开始执行cmd:"+cmd);
    	Session session = null;
        try {  
            if(login()){  
                session= conn.openSession();//打开一个会话  
                session.execCommand(cmd);//执行命令 
        		session.waitForCondition(ChannelCondition.EXIT_STATUS, 0);
                //获取错误信息
                Integer exitStatus = session.getExitStatus();
                if(exitStatus!=null&&exitStatus == 0) {//没有错误信息且退出码是0 说明执行成功
						logger.info("cmd:"+cmd+"执行成功");
						return true;
					}else {
						logger.info(cmd+"执行错误,返回码:"+exitStatus);
						return false;
					}
				}
        } catch (IOException e) {  
            e.printStackTrace(); 
        }  finally {
        	if (conn != null) {
				conn.close();
			}
        	if (session != null) {
        		session.close(); 
			}
             
        }
        logger.info("cmd:"+cmd+"执行结束");
        return false;
    }  
      
    
   /** 
    * 解析脚本执行返回的结果集 
    * @author Ickes 
    * @param in 输入流对象 
    * @param charset 编码 
    * @since V0.1 
    * @return 
    *       以纯文本的格式返回 
    */  
    private String processStdout(InputStream in, String charset){  
        InputStream    stdout = new StreamGobbler(in);  
        StringBuffer buffer = new StringBuffer();
        BufferedReader br = null;
        try {  
            br = new BufferedReader(new InputStreamReader(stdout,charset));  
            String line=null;  
            while((line=br.readLine()) != null){  
                buffer.append(line+"\n");  
            }  
        } catch (UnsupportedEncodingException e) {  
            e.printStackTrace();  
        } catch (IOException e) {  
            e.printStackTrace();  
        } finally {
			if (null != br) {
				try {
					br.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
        return buffer.toString();  
    }  
      
    public static void setCharset(String charset) {  
        DEFAULTCHART = charset;  
    }  
    public Connection getConn() {  
        return conn;  
    }  
    public void setConn(Connection conn) {  
        this.conn = conn;  
    }  
    public String getIp() {  
        return ip;  
    }  
    public void setIp(String ip) {  
        this.ip = ip;  
    }  
    public String getUserName() {  
        return userName;  
    }  
    public void setUserName(String userName) {  
        this.userName = userName;  
    }  
    public String getUserPwd() {  
        return userPwd;  
    }  
    public void setUserPwd(String userPwd) {  
        this.userPwd = userPwd;  
    }  
    
    public static void main(String[] args) {
    	/*System.out.println("execute begin "); 
        RemoteExecuteCommand rec=new RemoteExecuteCommand("10.0.53.56", "root",  
                "!QAZ2wsx3edc");  
        String result=rec.execute("sh /root/why/hht_test.sh 11");
        System.out.println(result.startsWith("0#"));
        if(result.startsWith("0#")) {
        	 System.out.println("execute ok! result="+result); 
        }else {
        	 System.out.println("execute error! result="+result);
        }*/
    	
    	//RemoteExecuteCommand rec=new RemoteExecuteCommand("192.168.56.101", "sml", "smlhao");  
    	//System.err.println(rec.isExecuteSuccess("/home/sml/checkAgentRun.sh 11"));
    	
		RemoteExecuteCommand rec = new RemoteExecuteCommand("192.168.204.123", "root", "123");
		String s = rec.execute("sh /home/harry/test/test.sh");
		System.out.println(s);
		String state = "UNKNOWN";
		String finalState = "UNKNOWN";
		if(StringUtils.isNotBlank(s)) {
			String enterLine = "\n";
			int a = 1;
			if (s.indexOf("\r\n") > -1) {
				enterLine = "\r\n";
				a = 2;
			} else if (s.indexOf("\r") > -1) {
				enterLine = "\r";
			} else if (s.indexOf("\n") > -1) {
				enterLine = "\n";
			}
			boolean flag = true;
			while (flag) {
				int i1 = s.indexOf(enterLine);
				String tmp = s.substring(0, i1);
				if(tmp.contains("State")) {
					String[] ss = tmp.split(":");
					state = ss[1].trim();
				} else if(tmp.contains("Final-State")) {
					String[] ss = tmp.split(":");
					finalState = ss[1].trim();
					break;
				} else {
					s = s.substring(i1 + a, s.length());
				}
			}
			System.err.println(s);
		}
		
		
		
    	/*String fn = "c.b.a.txt";
    	int idx = fn.lastIndexOf(".");
    	System.out.println(fn.substring(idx+1, fn.length()));*/
    }
    
}  