/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.jz.dmp.cmdexectool.scheduler.common.task.docTrans;

import com.alibaba.fastjson.JSONObject;
import com.jz.dmp.cmdexectool.common.constant.CommConstant;
import com.jz.dmp.cmdexectool.common.utils.FreeMarkerUtils;
import com.jz.dmp.cmdexectool.scheduler.common.process.ResourceInfo;
import com.jz.dmp.cmdexectool.scheduler.common.task.AbstractParameters;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.ui.freemarker.FreeMarkerConfigurationFactoryBean;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;

/**
 * Sql/Hql parameter
 */
public class DoctransParameters extends AbstractParameters {
	
	/**
	 * shell script
	 */
	private String script;

	private String taskAppId;
	
	/**
	 * resource list
	 */
	private List<ResourceInfo> resourceList;
	
	/**
	 * command需要执行的脚本
	 */
	private String cmdScript;

	public DoctransParameters() {
		
	}
	
	public DoctransParameters(String script, FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean) {
		this.script = script;
		
		JSONObject scriptObj = JSONObject.parseObject(script);
		
		Map<String, String> doctransModel = new HashMap<String, String>();
		
		String srcDir = scriptObj.getString("ftpUrl");
		String[] srcDirSuffix = srcDir.split(".");
		String fileSuffix = srcDirSuffix[1];
		
		String desDir = scriptObj.getString("ftpUrl");
		String sourceConvert = scriptObj.getString("ftpUrl");
		String sinkConvert = scriptObj.getString("ftpUrl");

		doctransModel.put("src_dir", srcDir);
		doctransModel.put("des_dir", desDir);
		doctransModel.put("source_convert", sourceConvert);
		doctransModel.put("sink_convert", sinkConvert);
		doctransModel.put("file_suffix", fileSuffix);
		this.cmdScript = FreeMarkerUtils.freemakerNoneWebJson(CommConstant.FTL_DOCTRANS, doctransModel, freeMarkerConfigurationFactoryBean);
		
	}

	public String getScript() {
		return script;
	}

	public void setScript(String script) {
		this.script = script;
	}

	public String getTaskAppId() {
		return taskAppId;
	}

	public void setTaskAppId(String taskAppId) {
		this.taskAppId = taskAppId;
	}
	
	public List<ResourceInfo> getResourceList() {
		return resourceList;
	}

	public void setResourceList(List<ResourceInfo> resourceList) {
		this.resourceList = resourceList;
	}

	public String getCmdScript() {
		return cmdScript;
	}

	public void setCmdScript(String cmdScript) {
		this.cmdScript = cmdScript;
	}

	@Override
	public boolean checkParameters() {
		return cmdScript != null && !cmdScript.isEmpty();
	}

	@Override
	public List<ResourceInfo> getResourceFilesList() {
		return resourceList;
	}

}
