/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.jz.dmp.cmdexectool.scheduler.common.task.ftp;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.ui.freemarker.FreeMarkerConfigurationFactoryBean;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;

import com.alibaba.fastjson.JSONObject;
import com.jz.dmp.cmdexectool.common.constant.CommConstant;
import com.jz.dmp.cmdexectool.common.utils.FreeMarkerUtils;
import com.jz.dmp.cmdexectool.scheduler.common.process.ResourceInfo;
import com.jz.dmp.cmdexectool.scheduler.common.task.AbstractParameters;

/**
 * Sql/Hql parameter
 */
public class FtpParameters extends AbstractParameters {
	
	private static Logger logger = LoggerFactory.getLogger(FtpParameters.class);
	
	/**
	 * shell script
	 */
	private String script;

	private String taskAppId;
	
	/**
	 * resource list
	 */
	private List<ResourceInfo> resourceList;
	
	/**
	 * command需要执行的脚本
	 */
	private String cmdScript;

	public FtpParameters() {
		
	}

	public FtpParameters(String script, FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean) {
		this.script = script;
		
		JSONObject scriptObj = JSONObject.parseObject(script);
		
		Map<String, String> ftpModel = new HashMap<String, String>();
		
		String ftpUrl = scriptObj.getString("ftpUrl");
		String[] ftpUrlPort = ftpUrl.split(":");
		String ip = ftpUrlPort[0];
		String port = "22";
		if (ftpUrlPort.length>1) {
			port = ftpUrlPort[1];
		}
		String username = scriptObj.getString("ftpUsername");
		String password = scriptObj.getString("ftpPassword");
		String srcDir = scriptObj.getString("ftpSourceFileDir");
		
		String ftpSaveDestDir = scriptObj.getString("ftpSaveDestDir");
		String[] desDirFileSuffix = ftpSaveDestDir.split("\\.");
		String desDir = desDirFileSuffix[0];
		String fileSuffix = "";
		if (desDirFileSuffix.length>1) {
			fileSuffix = desDirFileSuffix[1];
		}

		ftpModel.put("ip", ip);
		ftpModel.put("port", port);
		ftpModel.put("username", username);
		ftpModel.put("password", password);
		ftpModel.put("src_dir", srcDir);
		ftpModel.put("des_dir", desDir);
		ftpModel.put("file_suffix", fileSuffix);
		
		logger.info("调用ftp command模板参数【{}】", JSONObject.toJSONString(ftpModel));
		
		this.cmdScript = FreeMarkerUtils.freemakerNoneWebJson(CommConstant.FTL_SFTP_DOWNLOAD, ftpModel, freeMarkerConfigurationFactoryBean);
		
	}

	public String getScript() {
		return script;
	}

	public void setScript(String script) {
		this.script = script;
	}

	public String getTaskAppId() {
		return taskAppId;
	}

	public void setTaskAppId(String taskAppId) {
		this.taskAppId = taskAppId;
	}
	
	public List<ResourceInfo> getResourceList() {
		return resourceList;
	}

	public void setResourceList(List<ResourceInfo> resourceList) {
		this.resourceList = resourceList;
	}

	public String getCmdScript() {
		return cmdScript;
	}

	public void setCmdScript(String cmdScript) {
		this.cmdScript = cmdScript;
	}

	@Override
	public boolean checkParameters() {
		return cmdScript != null && !cmdScript.isEmpty();
	}

	@Override
	public List<ResourceInfo> getResourceFilesList() {
		return resourceList;
	}

}
