/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.jz.dmp.cmdexectool.scheduler.common.utils;

import com.jz.dmp.cmdexectool.common.constant.CommConstant;
import com.jz.dmp.cmdexectool.scheduler.common.enums.TaskType;

public class EnumUtils {

    private EnumUtils() {
        throw new UnsupportedOperationException("Construct EnumUtils");
    }

    public static <E extends Enum<E>> E getEnum(final Class<E> enumClass, final String enumName) {
        if (enumName == null) {
            return null;
        }
        try {
            return Enum.valueOf(enumClass, enumName);
        } catch (final IllegalArgumentException ex) {
            return null;
        }
    }

    public static <E extends Enum<E>> boolean isValidEnum(final Class<E> enumClass, final String enumName) {
        if (enumName == null) {
            return false;
        }
        try {
            Enum.valueOf(enumClass, enumName);
            return true;
        } catch (final IllegalArgumentException ex) {
            return false;
        }
    }

	/**  
	* @Title: getTaskType  
	* @Description: TODO(将任务类型转成枚举)  
	* @param @param taskType
	* @param @return    参数  
	* @return TaskType    返回类型  
	* @throws  
	*/ 
	public static TaskType getTaskType(String taskType) {
		if (taskType.equals(CommConstant.WORK_TYPE_SHELL)) {
			return TaskType.SHELL;
		}else if (taskType.equals(CommConstant.WORK_TYPE_SQL)) {
			return TaskType.SQL;
		}else if (taskType.equals(CommConstant.WORK_TYPE_FTP)) {
			return TaskType.FTP;
		}else if (taskType.equals(CommConstant.WORK_TYPE_UNZIPFILE)) {
			return TaskType.UNZIPFILE;
		}else if (taskType.equals(CommConstant.WORK_TYPE_DOCTRANS)) {
			return TaskType.DOCTRANS;
		}else if (taskType.equals(CommConstant.WORK_TYPE_HDFS)) {
			return TaskType.HDFS;
		}else if (taskType.equals(CommConstant.WORK_TYPE_SYNC)) {
            return TaskType.SYNC;
        }
		return null;
	}
}
