/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.jz.dmp.cmdexectool.scheduler.service.process;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.ui.freemarker.FreeMarkerConfigurationFactoryBean;
import org.springframework.util.CollectionUtils;
//import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.jz.dmp.cmdexectool.common.constant.CommConstant;
import com.jz.dmp.cmdexectool.controller.bean.DmpDevelopTaskDto;
import com.jz.dmp.cmdexectool.controller.bean.DmpDevelopTaskHistoryDto;
import com.jz.dmp.cmdexectool.controller.bean.DmpProjectConfigInfoDto;
import com.jz.dmp.cmdexectool.controller.bean.MyDmpDevelopTaskHistoryConverter;
import com.jz.dmp.cmdexectool.mapper.DmpDevelopTaskHistoryMapper;
import com.jz.dmp.cmdexectool.mapper.DmpDevelopTaskMapper;
import com.jz.dmp.cmdexectool.mapper.DmpProjectConfigInfoMapper;
import com.jz.dmp.cmdexectool.mapper.DmpSyncingDatasourceDao;
import com.jz.dmp.cmdexectool.scheduler.common.Constants;
import com.jz.dmp.cmdexectool.scheduler.common.enums.ExecutionStatus;
import com.jz.dmp.cmdexectool.scheduler.common.enums.JobType;
import com.jz.dmp.cmdexectool.scheduler.common.task.docTrans.DoctransParameters;
import com.jz.dmp.cmdexectool.scheduler.common.task.ftp.FtpParameters;
import com.jz.dmp.cmdexectool.scheduler.common.task.hdfs.HdfsParameters;
import com.jz.dmp.cmdexectool.scheduler.common.task.shell.ShellParameters;
import com.jz.dmp.cmdexectool.scheduler.common.task.sql.SqlParameters;
import com.jz.dmp.cmdexectool.scheduler.common.task.sync.SyncParameters;
import com.jz.dmp.cmdexectool.scheduler.common.task.unzipfile.UnzipfileParameters;
import com.jz.dmp.cmdexectool.scheduler.server.entity.TaskExecutionContext;
import com.jz.dmp.cmdexectool.scheduler.server.worker.task.AbstractTask;
import com.jz.dmp.cmdexectool.scheduler.server.worker.task.TaskManager;

/**
 * process relative dao that some mappers in this.
 */
@Component
public class ProcessService {

	private final Logger logger = LoggerFactory.getLogger(getClass());
	
	@Value("${spring.public-key}")
    private String publicKey;

	@Autowired
	private DmpDevelopTaskMapper dmpDevelopTaskMapper;
	@Autowired
	private DmpDevelopTaskHistoryMapper dmpDevelopTaskHistoryMapper;
	@Autowired
	private DmpProjectConfigInfoMapper dmpProjectConfigInfoMapper;
	@Autowired
	private DmpSyncingDatasourceDao dmpSyncingDatasourceDao;
	//@Autowired
	//private FreeMarkerConfigurer freeMarkerConfigurer;
	@Autowired
	private FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean;

	/**
	 * @Title: taskStart @Description: TODO(启动task) @param 参数 @return void
	 *         返回类型 @throws
	 */
	public void taskStart(Integer taskId, String jobId, boolean isSingle, String execId) {
		try {
			TaskExecutionContext taskExecutionContext = findTaskExecutionContextById(taskId, jobId, isSingle, execId);
			AbstractTask task = TaskManager.newTask(taskExecutionContext);

			// AbstractTask task = null;
			logger.info("task info : {}", task);

			// job init
			task.init();

			// job handle
			task.handle();
			ExecutionStatus status = ExecutionStatus.SUCCESS;

			if (task.getExitStatusCode() == Constants.EXIT_CODE_SUCCESS) {
				status = ExecutionStatus.SUCCESS;
			} else if (task.getExitStatusCode() == Constants.EXIT_CODE_KILL) {
				status = ExecutionStatus.KILL;
			} else {
				status = ExecutionStatus.FAILURE;
			}

			logger.info(status.toString());
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			logger.info(e.getMessage());
		}
	}

	/**
	 * @throws Exception @Title: findTaskExecutionContextById @Description:
	 *                   TODO(根据taskId,jobId等信息将job转成taskExecutionContext) @param @param
	 *                   taskId @param @param jobId @param @param
	 *                   isSingle @param @return 参数 @return TaskExecutionContext
	 *                   返回类型 @throws
	 */
	public TaskExecutionContext findTaskExecutionContextById(Integer taskId, String jobId, boolean isSingle, String execId)
			throws Exception {
		TaskExecutionContext taskExecutionContext = new TaskExecutionContext();
		// 获取任务数据，并获取job json，并组装TaskInstance
		DmpDevelopTaskDto dmpDevelopTaskDto = dmpDevelopTaskMapper.findById(taskId);
		if (!isSingle) {
			Map<String, Object> param = new HashMap<String, Object>();
			param.put("taskId", dmpDevelopTaskDto.getId());
			param.put("version", dmpDevelopTaskDto.getPublishVersion());
			List<DmpDevelopTaskHistoryDto> dmpDevelopTaskHistoryDtos = dmpDevelopTaskHistoryMapper.findList(param);
			if (CollectionUtils.isEmpty(dmpDevelopTaskHistoryDtos)) {
				throw new RuntimeException("没有找到对应的历史版本");
			}
			dmpDevelopTaskDto = MyDmpDevelopTaskHistoryConverter.INSTANCE().historyDto2taskDto(dmpDevelopTaskHistoryDtos.get(0));
		}

		taskExecutionContext = dto2execcontext(dmpDevelopTaskDto, jobId);
		//设置azkaban执行ID
		taskExecutionContext.setExecId(execId);

		return taskExecutionContext;
	}
	
	/**  
	* @Title: dto2execcontext  
	* @Description: TODO(将task的job转换成taskExecutionContext)  
	* @param @param dmpDevelopTaskDto
	* @param @param jobId
	* @param @return    参数  
	* @return TaskExecutionContext    返回类型  
	* @throws  
	*/ 
	private TaskExecutionContext dto2execcontext(DmpDevelopTaskDto dmpDevelopTaskDto, String jobId)throws Exception {
		TaskExecutionContext taskExecutionContext = new TaskExecutionContext();
		
		Map<String, Object> param =new HashMap<String, Object>();
		param.put("projectId", dmpDevelopTaskDto.getProjectId());
		List<DmpProjectConfigInfoDto> dtos = dmpProjectConfigInfoMapper.findList(param);
		if (CollectionUtils.isEmpty(dtos)) {
			logger.info("项目【{}】没有配置信息，请联系管理员", dmpDevelopTaskDto.getProjectId());
			throw new RuntimeException("项目没有配置信息，请联系管理员");
		}
		
		DmpProjectConfigInfoDto projectConfigInfoDto = dtos.get(0);
		
		JobType jobType = null;
		String script = "";
		String taskAppId = "";
		
		String taskName = dmpDevelopTaskDto.getName();
		
		String taskType = dmpDevelopTaskDto.getTaskType();//任务类型
		if (taskType.equals(CommConstant.TASK_TYPE_DEVSHELL)) {
			jobType = JobType.shell;
			script = dmpDevelopTaskDto.getScript();
			taskAppId = taskName;
		}else if (taskType.equals(CommConstant.TASK_TYPE_DEVSQL)) {
			jobType = JobType.sql;
			script = dmpDevelopTaskDto.getScript();
			taskAppId = taskName;
		}else if (taskType.equals(CommConstant.TASK_TYPE_OFFLINE)) {
			jobType = JobType.sync;
			script = dmpDevelopTaskDto.getScript();
			taskAppId = taskName;
		}else if (taskType.equals(CommConstant.TASK_TYPE_DEVELOP)) {
			String scriptStr = dmpDevelopTaskDto.getScript();
			JSONObject scriptObj = JSONObject.parseObject(scriptStr);
			JSONArray array = scriptObj.getJSONArray("nodes");
			for (int i = 0; i < array.size(); i++) {
				JSONObject jObject = array.getJSONObject(i);
				String taskJobType = jObject.getString("taskType");
				String lable = jObject.getString("label");
				String comparLabel = lable;
				if (!CommConstant.WORK_TYPE_START.equals(taskJobType) && !CommConstant.WORK_TYPE_STOP.equals(taskJobType)) {
					comparLabel = lable+"_"+taskName;
				}
				if (jobId.equals(comparLabel)) {
					jobType = JobType.valueOf(jObject.getString("taskType"));//job类型
					script = jObject.toJSONString();
					taskAppId = lable;
					break;
				}
			}
		}
		
		switch (jobType) {
		case shell:
			ShellParameters shellParameters = new ShellParameters();
			shellParameters.setScript(script);
			shellParameters.setTaskAppId(taskAppId);
			taskExecutionContext = new TaskExecutionContext(shellParameters, projectConfigInfoDto);
			
			break;
		case sql:
			SqlParameters sqlParameters = new SqlParameters(script, projectConfigInfoDto, dmpSyncingDatasourceDao, freeMarkerConfigurationFactoryBean, publicKey);
			sqlParameters.setTaskAppId(taskAppId);
			taskExecutionContext = new TaskExecutionContext(sqlParameters, projectConfigInfoDto);
			
			break;
		case sync:
			SyncParameters sync = new SyncParameters(script, projectConfigInfoDto, dmpSyncingDatasourceDao, freeMarkerConfigurationFactoryBean, publicKey);
			sync.setTaskAppId(taskAppId);
			taskExecutionContext = new TaskExecutionContext(sync, projectConfigInfoDto);
			break;
		case subprocess:
			break;
		case ftp:
			FtpParameters ftpParameters = new FtpParameters(script, freeMarkerConfigurationFactoryBean);
			ftpParameters.setTaskAppId(taskAppId);
			taskExecutionContext = new TaskExecutionContext(ftpParameters, projectConfigInfoDto);
			break;
		case unzipFile:
			UnzipfileParameters unzipfileParameters = new UnzipfileParameters(script, freeMarkerConfigurationFactoryBean);
			unzipfileParameters.setScript(script);
			unzipfileParameters.setTaskAppId(taskAppId);
			taskExecutionContext = new TaskExecutionContext(unzipfileParameters, projectConfigInfoDto);
			break;
		case docTrans:
			DoctransParameters doctransParameters = new DoctransParameters(script, freeMarkerConfigurationFactoryBean);
			doctransParameters.setScript(script);
			doctransParameters.setTaskAppId(taskAppId);
			taskExecutionContext = new TaskExecutionContext(doctransParameters, projectConfigInfoDto);
			break;
		case hdfs:
			HdfsParameters hdfsParameters = new HdfsParameters(script, freeMarkerConfigurationFactoryBean);
			hdfsParameters.setScript(script);
			hdfsParameters.setTaskAppId(taskAppId);
			taskExecutionContext = new TaskExecutionContext(hdfsParameters, projectConfigInfoDto);
			break;

		default:
			break;
		}
		
		return taskExecutionContext;
	}

}
