/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.jz.cmdexectool.test.task.shell;

import static com.jz.dmp.cmdexectool.scheduler.common.Constants.HIVE_CONF;
import static com.jz.dmp.cmdexectool.scheduler.common.Constants.PASSWORD;
import static com.jz.dmp.cmdexectool.scheduler.common.Constants.SEMICOLON;
import static com.jz.dmp.cmdexectool.scheduler.common.Constants.USER;
import static com.jz.dmp.cmdexectool.scheduler.common.enums.DbType.HIVE;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.jz.dmp.cmdexectool.ApiApplication;
import com.jz.dmp.cmdexectool.scheduler.common.Constants;
import com.jz.dmp.cmdexectool.scheduler.common.enums.DbType;
import com.jz.dmp.cmdexectool.scheduler.common.enums.ExecutionStatus;
import com.jz.dmp.cmdexectool.scheduler.common.enums.MyDbType;
import com.jz.dmp.cmdexectool.scheduler.common.process.Property;
import com.jz.dmp.cmdexectool.scheduler.common.task.sql.SqlBinds;
import com.jz.dmp.cmdexectool.scheduler.common.task.sql.SqlType;
import com.jz.dmp.cmdexectool.scheduler.common.utils.CollectionUtils;
import com.jz.dmp.cmdexectool.scheduler.common.utils.CommonUtils;
import com.jz.dmp.cmdexectool.scheduler.common.utils.JSONUtils;
import com.jz.dmp.cmdexectool.scheduler.common.utils.ParameterUtils;
import com.jz.dmp.cmdexectool.scheduler.dao.datasource.DataSourceFactory;
import com.jz.dmp.cmdexectool.scheduler.dao.datasource.MyBaseDataSource;
import com.jz.dmp.cmdexectool.scheduler.dao.utils.DatabaseUtils;
import com.jz.dmp.cmdexectool.scheduler.server.entity.TaskExecutionContext;
import com.jz.dmp.cmdexectool.scheduler.server.utils.ParamUtils;
import com.jz.dmp.cmdexectool.scheduler.server.worker.task.AbstractTask;
import com.jz.dmp.cmdexectool.scheduler.server.worker.task.TaskManager;
import com.jz.dmp.cmdexectool.scheduler.service.process.ProcessService;

/**
 *  python shell command executor test
 */
@RunWith(SpringRunner.class)
@SpringBootTest(classes = ApiApplication.class, webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class SQLCommandExecutorTest {

    private static final Logger logger = LoggerFactory.getLogger(SQLCommandExecutorTest.class);
    
    @Autowired
    private ProcessService processService;
    
    private static final int LIMIT = 10000;
    
    @Test
    public void test22() {
    	System.out.println("test");
    }
    
    //@Test
    public void test() {
		try {
			TaskExecutionContext taskExecutionContext = processService.findTaskExecutionContextById(460,null,true, "2");
			AbstractTask task = TaskManager.newTask(taskExecutionContext);

	        logger.info("task info : {}", task);

	        // job init
	        task.init();

	        // job handle
	        task.handle();
	        ExecutionStatus status = ExecutionStatus.SUCCESS;

	        if (task.getExitStatusCode() == Constants.EXIT_CODE_SUCCESS){
	            status = ExecutionStatus.SUCCESS;
	        }else if (task.getExitStatusCode() == Constants.EXIT_CODE_KILL){
	            status = ExecutionStatus.KILL;
	        }else {
	            status = ExecutionStatus.FAILURE;
	        }

	        logger.info(status.toString());
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			logger.info(e.getMessage());
		}
    }
    
    //@Test
    public void testJdbcHandler() {
    	// set the name of the current thread
        //String threadLoggerInfoName = String.format(Constants.TASK_LOG_INFO_FORMAT, "test");
        //Thread.currentThread().setName(threadLoggerInfoName);

        try {
        	System.out.println("test-----------");
            // load class
            DataSourceFactory.loadClass(DbType.valueOf("MYSQL"));
            
            MyBaseDataSource myBaseDataSource = new MyBaseDataSource();
            myBaseDataSource.setJdbcUrlDirect("jdbc:mysql://192.168.1.140:3307/dmp_web_new");
            myBaseDataSource.setUser("dmp");
            myBaseDataSource.setPassword("Ioubuy123");
            myBaseDataSource.setMyDbType(MyDbType.MySQL);
            
            //String json = JSONObject.toJSONString(mySQLDataSource);

            // get datasource
            // BaseDataSource baseDataSource = DataSourceFactory.getDatasource(DbType.valueOf("MYSQL"), json);
            
			/*
			 * BaseDataSource baseDataSource = mySQLDataSource;
			 * baseDataSource.s(DbType.MYSQL.name());
			 * baseDataSource.setJdbcUrlDirect("jdbc:mysql://192.168.1.140:3307/dmp_web_new"
			 * ); baseDataSource.setUser("dmp"); baseDataSource.setPassword("Ioubuy123");
			 */
            
            // ready to execute SQL and parameter entity Map
            SqlBinds mainSqlBinds = getSqlAndSqlParamsMap("insert into test(id, name) values(1, 'test')");
            List<String> preStatements = new ArrayList<String>();
            preStatements.add("insert into test(id, name) values(1, 'test')");
            preStatements.add("insert into test(id, name) values(2, 'test2')");
            
            List<SqlBinds> preStatementSqlBinds = Optional.ofNullable(preStatements)
                    .orElse(new ArrayList<>())
                    .stream()
                    .map(x ->{
                    	return getSqlAndSqlParamsMap(x);
                    }).collect(Collectors.toList());
            
            
            List<SqlBinds> postStatementSqlBinds = Optional.ofNullable(preStatements)
                    .orElse(new ArrayList<>())
                    .stream()
                    .map(this::getSqlAndSqlParamsMap)
                    .collect(Collectors.toList());
			
            //List<String> createFuncs = UDFUtils.createFuncs(sqlTaskExecutionContext.getUdfFuncTenantCodeMap(),
             //       logger);
            
            List<String> createFuncs = new ArrayList<String>();
            // execute sql task
            //DatabaseUtils.executeFuncAndSql(mainSqlBinds, preStatementSqlBinds, postStatementSqlBinds, createFuncs);
            
            DatabaseUtils.executeUpdateSql(preStatementSqlBinds, myBaseDataSource);

            //setExitStatusCode(Constants.EXIT_CODE_SUCCESS);
        } catch (Exception e) {
            //setExitStatusCode(Constants.EXIT_CODE_FAILURE);
            logger.error("sql task error", e);
            //throw e;
        }
    }
        
        /**
         * ready to execute SQL and parameter entity Map
         * @return SqlBinds
         */
        private SqlBinds getSqlAndSqlParamsMap(String sql) {
            Map<Integer,Property> sqlParamsMap =  new HashMap<>();
            StringBuilder sqlBuilder = new StringBuilder();
            
            // find process instance by task id

            /*
            Map<String, Property> paramsMap = ParamUtils.convert(ParamUtils.getUserDefParamsMap(taskExecutionContext.getDefinedParams()),
                    taskExecutionContext.getDefinedParams(),
                    sqlParameters.getLocalParametersMap(),
                    CommandType.of(taskExecutionContext.getCmdTypeIfComplement()),
                    taskExecutionContext.getScheduleTime());
*/
            Map<String, Property> paramsMap = new HashMap<String, Property>();
            		
            // spell SQL according to the final user-defined variable
            if(paramsMap == null){
                sqlBuilder.append(sql);
                return new SqlBinds(sqlBuilder.toString(), sqlParamsMap);
            }

            if (StringUtils.isNotEmpty("")){
                String title = ParameterUtils.convertParameterPlaceholders("",
                        ParamUtils.convert(paramsMap));
                logger.info("SQL title : {}",title);
                //sqlParameters.setTitle(title);
            }
            
            //new
            //replace variable TIME with $[YYYYmmddd...] in sql when history run job and batch complement job
            sql = ParameterUtils.replaceScheduleTime(sql, new Date());
            // special characters need to be escaped, ${} needs to be escaped
            String rgex = "['\"]*\\$\\{(.*?)\\}['\"]*";
            setSqlParamsMap(sql, rgex, sqlParamsMap, paramsMap);

            // replace the ${} of the SQL statement with the Placeholder
            String formatSql = sql.replaceAll(rgex, "?");
            sqlBuilder.append(formatSql);

            // print repalce sql
            printReplacedSql(sql, formatSql, rgex, sqlParamsMap);
            return new SqlBinds(sqlBuilder.toString(), sqlParamsMap);
        }
        
        /**
         * print replace sql
         * @param content       content
         * @param formatSql     format sql
         * @param rgex          rgex
         * @param sqlParamsMap  sql params map
         */
        public void printReplacedSql(String content, String formatSql,String rgex, Map<Integer,Property> sqlParamsMap){
            //parameter print style
            logger.info("after replace sql , preparing : {}" , formatSql);
            StringBuilder logPrint = new StringBuilder("replaced sql , parameters:");
            for(int i=1;i<=sqlParamsMap.size();i++){
                logPrint.append(sqlParamsMap.get(i).getValue()+"("+sqlParamsMap.get(i).getType()+")");
            }
            logger.info("Sql Params are {}", logPrint);
        }
        
        /**
         * regular expressions match the contents between two specified strings
         * @param content           content
         * @param rgex              rgex
         * @param sqlParamsMap      sql params map
         * @param paramsPropsMap    params props map
         */
        public void setSqlParamsMap(String content, String rgex, Map<Integer,Property> sqlParamsMap, Map<String,Property> paramsPropsMap){
            Pattern pattern = Pattern.compile(rgex);
            Matcher m = pattern.matcher(content);
            int index = 1;
            while (m.find()) {

                String paramName = m.group(1);
                Property prop =  paramsPropsMap.get(paramName);

                sqlParamsMap.put(index,prop);
                index ++;
            }
        }
        
        /**
         * execute function and sql
         * @param mainSqlBinds          main sql binds
         * @param preStatementsBinds    pre statements binds
         * @param postStatementsBinds   post statements binds
         * @param createFuncs           create functions
         */
        public void executeFuncAndSql(SqlBinds mainSqlBinds,
                                            List<SqlBinds> preStatementsBinds,
                                            List<SqlBinds> postStatementsBinds,
                                            List<String> createFuncs){
            Connection connection = null;
            PreparedStatement stmt = null;
            ResultSet resultSet = null;
            try {
                // if upload resource is HDFS and kerberos startup
                CommonUtils.loadKerberosConf();
                // create connection
                connection = createConnection();
                // create temp function
                if (CollectionUtils.isNotEmpty(createFuncs)) {
                    createTempFunction(connection,createFuncs);
                }

                // pre sql
                preSql(connection,preStatementsBinds);
                stmt = prepareStatementAndBind(connection, mainSqlBinds);

                // decide whether to executeQuery or executeUpdate based on sqlType
                if (1 == SqlType.QUERY.ordinal()) {
                    // query statements need to be convert to JsonArray and inserted into Alert to send
                    resultSet = stmt.executeQuery();
                    resultProcess(resultSet);

                } else if (1 == SqlType.NON_QUERY.ordinal()) {
                    // non query statement
                    stmt.executeUpdate();
                }

                postSql(connection,postStatementsBinds);

            } catch (Exception e) {
                logger.error("execute sql error",e);
                throw new RuntimeException("execute sql error");
            } finally {
                close(resultSet,stmt,connection);
            }
        }
        
        /**
         *  close jdbc resource
         *
         * @param resultSet resultSet
         * @param pstmt pstmt
         * @param connection connection
         */
        private void close(ResultSet resultSet,
                           PreparedStatement pstmt,
                           Connection connection){
            if (resultSet != null){
                try {
                    resultSet.close();
                } catch (SQLException e) {
                    logger.error("close result set error : {}",e.getMessage(),e);
                }
            }

            if (pstmt != null){
                try {
                    pstmt.close();
                } catch (SQLException e) {
                    logger.error("close prepared statement error : {}",e.getMessage(),e);
                }
            }

            if (connection != null){
                try {
                    connection.close();
                } catch (SQLException e) {
                    logger.error("close connection error : {}",e.getMessage(),e);
                }
            }
        }
        
        /**
         * post sql
         *
         * @param connection connection
         * @param postStatementsBinds postStatementsBinds
         * @throws Exception
         */
        private void postSql(Connection connection,
                             List<SqlBinds> postStatementsBinds) throws Exception{
            for (SqlBinds sqlBind: postStatementsBinds) {
                try (PreparedStatement pstmt = prepareStatementAndBind(connection, sqlBind)){
                    int result = pstmt.executeUpdate();
                    logger.info("post statement execute result: {},for sql: {}",result,sqlBind.getSql());
                }
            }
        }
        
        /**
         * result process
         *
         * @param resultSet resultSet
         * @throws Exception Exception
         */
        private void resultProcess(ResultSet resultSet) throws Exception{
            ArrayNode resultJSONArray = JSONUtils.createArrayNode();
            ResultSetMetaData md = resultSet.getMetaData();
            int num = md.getColumnCount();

            int rowCount = 0;

            while (rowCount < LIMIT && resultSet.next()) {
                ObjectNode mapOfColValues = JSONUtils.createObjectNode();
                for (int i = 1; i <= num; i++) {
                    mapOfColValues.set(md.getColumnLabel(i), JSONUtils.toJsonNode(resultSet.getObject(i)));
                }
                resultJSONArray.add(mapOfColValues);
                rowCount++;
            }
            String result = JSONUtils.toJsonString(resultJSONArray);
            logger.debug("execute sql : {}", result);

            //sendAttachment(StringUtils.isNotEmpty(sqlParameters.getTitle()) ?
              //              sqlParameters.getTitle(): taskExecutionContext.getTaskName() + " query result sets",
               //     JSONUtils.toJsonString(resultJSONArray));
        }
        
        /**
         *  pre sql
         *
         * @param connection connection
         * @param preStatementsBinds preStatementsBinds
         */
        private void preSql(Connection connection,
                            List<SqlBinds> preStatementsBinds) throws Exception{
            for (SqlBinds sqlBind: preStatementsBinds) {
                try (PreparedStatement pstmt = prepareStatementAndBind(connection, sqlBind)){
                    int result = pstmt.executeUpdate();
                    logger.info("pre statement execute result: {}, for sql: {}",result,sqlBind.getSql());

                }
            }
        }
        
        /**
         * preparedStatement bind
         * @param connection connection
         * @param sqlBinds  sqlBinds
         * @return PreparedStatement
         * @throws Exception Exception
         */
        private PreparedStatement prepareStatementAndBind(Connection connection, SqlBinds sqlBinds) throws Exception {
            // is the timeout set
            //boolean timeoutFlag = TaskTimeoutStrategy.of(taskExecutionContext.getTaskTimeoutStrategy()) == TaskTimeoutStrategy.FAILED ||
            //        TaskTimeoutStrategy.of(taskExecutionContext.getTaskTimeoutStrategy()) == TaskTimeoutStrategy.WARNFAILED;
            PreparedStatement stmt = connection.prepareStatement(sqlBinds.getSql());
            //if(timeoutFlag){
            //    stmt.setQueryTimeout(taskExecutionContext.getTaskTimeout());
            //}
            Map<Integer, Property> params = sqlBinds.getParamsMap();
            if(params != null) {
                for (Map.Entry<Integer, Property> entry : params.entrySet()) {
                    Property prop = entry.getValue();
                    ParameterUtils.setInParameter(entry.getKey(), stmt, prop.getType(), prop.getValue());
                }
            }
            logger.info("prepare statement replace sql : {} ", stmt);
            return stmt;
        }
        
        /**
         * create temp function
         *
         * @param connection connection
         * @param createFuncs createFuncs
         * @throws Exception
         */
        private void createTempFunction(Connection connection,
                                        List<String> createFuncs) throws Exception{
            try (Statement funcStmt = connection.createStatement()) {
                for (String createFunc : createFuncs) {
                    logger.info("hive create function sql: {}", createFunc);
                    funcStmt.execute(createFunc);
                }
            }
        }
        
        /**
         * create connection
         *
         * @return connection
         * @throws Exception Exception
         */
        private Connection createConnection() throws Exception{
            // if hive , load connection params if exists
            Connection connection = null;
            String type = "MYSQL";
            if (HIVE == DbType.valueOf(type)) {
                Properties paramProp = new Properties();
                paramProp.setProperty(USER, "");
                paramProp.setProperty(PASSWORD, "");
                Map<String, String> connParamMap = CollectionUtils.stringToMap("",
                        SEMICOLON,
                        HIVE_CONF);
                paramProp.putAll(connParamMap);

                connection = DriverManager.getConnection("",
                        paramProp);
            }else{
            	connection = DriverManager.getConnection("jdbc:mysql://192.168.1.140:3307/dmp_web_new?characterEncoding=utf8&useSSL=false",
                        "dmp",
                        "Ioubuy123");
            }
            return connection;
        }
}